"""Handle processes."""

import subprocess

from rwx import txt


class Command:
    """Command to run."""

    def __init__(self, *arguments: str | tuple[str, ...]) -> None:
        """Set raw & flat arguments."""
        self.raw = arguments
        self.flat: list[str] = []

    def __repr__(self) -> str:
        """Represent command."""
        return f"Command({self.raw!r})"

    def __str__(self) -> str:
        """Return raw & flat."""
        return f"""\
 raw = {self.raw}
flat = {self.flat}
"""


def get_tuples_args(*items: str | tuple[str, ...]) -> list[str]:
    """Turn arguments tuples into an arguments list."""
    args: list[str] = []
    for item in items:
        match item:
            case str():
                args.append(item)
            case tuple():
                args.extend(item)
    return args


def run(*items: str | tuple[str, ...]) -> subprocess.CompletedProcess:
    """Run from a list of arguments tuples."""
    return subprocess.run(
        get_tuples_args(*items), capture_output=False, check=True
    )


def run_line(*items: str | tuple[str, ...], charset: str = txt.CHARSET) -> str:
    """Run and return output line."""
    line, *_ = run_lines(*items, charset=charset)
    return line


def run_lines(
    *items: str | tuple[str, ...], charset: str = txt.CHARSET
) -> list[str]:
    """Run and return output lines."""
    process = subprocess.run(
        get_tuples_args(*items), capture_output=True, check=True
    )
    string = process.stdout.decode(charset)
    return string.rstrip().splitlines()
